function [x,F,k,fe,flag, xlist, Flist] = ASFW(Fun,L,A,a0,tol,maxit,opt,print)
%% Away Step Frank Wolfe Algorithm for Multiobjective Optimiztion 
%
%
% min  F(x)
% s.t. x in conv(A)
%
% Syntax: 
%       [x,F,k,fe] = ASFW(Fun,L,A,a0,tol,maxit,opt,print)
%
% Input: 
%         Fun: vectorial function (must output F and its Jacobian)
%         L: Lipschitz constant
%         A: set of p points in R^n
%         a0: coeffcients of initial point in A (x0 = A*x0)
%         tol: tolerance for stopping criteria
%         maxit: Maximum number of iterations
%         opt: 0 - FW, 1 - ASFW
%         print: > 0 print iteration info
%
% Output: 
%         x: a weak Pareto point
%         F: F(x)
%         k: iteration counter
%         fe: number of function evaluations
%         flag: output flag
%
%% Initialization
    flag = -2;
    [n, p] = size(A);
    a = a0;
    k = 0; fe = 0;
    x = A*a0;
    if nargout > 5
        xlist = x';
    end
    opts = optimset('Display','off');
    
    F = Fun(x);
    Flist = F;
    %tau = 2;
    tolp = 1e-14;

%% main loop  
while 1             
    U = find(a > tolp);
    pu = length(U);
    q = pu;
    
    [~, J] = Fun(x);
    
    if k==0
       m = length(F);
    end
    fe = fe + 1;
    
    %% Frank-Wolfe subproblem (1)
    JA = J*A;
    Jx = J*x;
    [sol,FVAL,EXITFLAG,OUTPUT,LAMBDA] = ...
        linprog([zeros(p,1); 1],[JA -ones(m,1)],Jx,[ones(1,p) 0],1,[zeros(p,1); -Inf],[ones(p,1); 0],[],opts);
    as = sol(1:end-1);
    xb = A*as;
    dfw = xb - x;
    
    theta = sol(end); 
    
    % stopping criterion
    if abs(theta) < tol || theta > 0
        flag = 0;
        return
    end
    
    %% Away subproblem (Type 1)
    if opt
        [~, idxu] = max(min(JA(:,U) - Jx*ones(1,pu),[],1));
        ku = U(idxu);
        w = A(:,ku);

        da = x - w;
        thetaw = max(J*da);

        %fprintf('k: %d, thetafw: %g, thetaaw: %g \n',k,theta,thetaw)
    end
    
    %% Choose the best direction between (1) and (2)
    away = 0;

    % AWAY 
    if ~opt || theta <= thetaw
        amax = 1;
        d = dfw;                       %Direction
        an = as;
        gamma = min(amax, -theta/(L*norm(d,2)^2)); %Adaptive step-size
        %gamma = 2/(k+2); % Diminishing step-size
    else                                    %Choose Away Step 
        away=1;
        amax = a(ku)/(1 - a(ku));             %to ensure the point remains on the convA
        d = da;                             %Direction
        gamma = min(amax, -thetaw/(L*norm(d,2)^2));  %Adaptive step-size
        %gamma = min(amax, 2/(k+2)); % Diminishing step-size
        an = 0*a; an(ku) = 1;
        theta = thetaw;
    end

%% ... for hypothesis testing ...
%     thetatil = 0;
%     if 0 && opt 
%         for l=1:p
%             Jx = J*A(:,l);
%             [sol,FVAL,EXITFLAG,OUTPUT,LAMBDA] = ...
%                 linprog([zeros(p,1); 1],[JA -ones(m,1)],Jx,[ones(1,p) 0],1,[zeros(p,1); -Inf],[],[],opts);
%             if sol(end) < thetatil
%                 thetatil = sol(end);
%             end
%         end
%     end
%% Update iterate x and coefficients a
    
    if print
        %fprintf('k:%d  nnz(a):%d , gmax: %g , gamma: %g, away: %d, beta: %g  \n', k, nnz(a), amax, gamma, away, thetatil/theta);
        fprintf('k:%d  nnz(a):%d , gmax: %g , gamma: %g, away: %d, theta: %g  \n', k, nnz(a), amax, gamma, away, theta);
    end
    
    x0 = x;
    x = x0 + gamma*d;

    [F, ~] = Fun(x);
    
    if nargout > 6
       Flist = [Flist F];
    end
    
    aold = a;
    
    if ~away
        a = (1-gamma)*a + gamma*an; %FW
    else
        a = (1+gamma)*a - gamma*an; 
    end
    
    % clean up tiny negative coefficients
    a(a<tolp) = 0;
    a = a./sum(a);
    
    if norm(a - aold,1) < 5*tol
        flag = 2;
        return 
    end
    
    k = k+1;
    
    if nargout > 5
        xlist = [xlist; x'];
    end
    
    if (k > maxit)
        fprintf('ASFW: maximum number of iterations reached!\n')
        flag=-1;
        return
    end
    
end


%%%%% OBLIVIUM %%%%%%

%    %% Backtracking for estimating L 
%    bt = 0;
%     while sum(F - Fold - gamma*theta*ones(m,1) > (L/2)*(gamma^2)*(norm(d,2)^2)*ones(m,1))>0
%         L = tau*L;
%         gamma = min(amax, -theta/(L*norm(d,2)^2));
%         x = x0 + gamma*d;
%         [F, ~] = Fun(x);
%         bt = 1;
%     end
%     
%     if bt
%         L = 0.5*L;
%     end
