function [x,F,k,fe,flag,xlist,tstop] = CSD(Fun,L,A,a0,tol,maxit)
%% Constrained/Projected Steepest Descent Algorithm for Multiobjective Optimiztion 
%
%
% min  F(x)
% s.t. x in conv(A)
%
% Syntax: 
%       [x,F,k,fe] = CSD(Fun,L,A,a0,tol,maxit)
%
% Input: 
%         Fun: vectorial function (must output F and its Jacobian)
%         A: set of p points in R^n
%         a0: coefficients for initial point in A: x0 = A*a0
%         tol: tolerance for stopping criteria
%         maxit: Maximum number of iterations
%
% Output: 
%         x: a weak Pareto point
%         F: F(x)
%         k: iteration counter
%         fe: number of function evaluations
%         flag: output flag
%
%% Initialization
    tstop = 0;
    flag = -2;
    [n, p] = size(A);
    a = a0;
    k = 0; fe = 0;
    x = A*a0;
    if nargout > 5
        xlist = x';
    end
    opts = optimset('Display','off');
    H = [A'*A zeros(p,1);zeros(1,p) 0];
    tolp = 1e-14;

%% main loop  
while 1                 
    [F, J] = Fun(x);
    if k==0
       m = length(F);
    end
    fe = fe + 1;
    
    %% Constrained SD subproblem (1)
    JA = J*A;
    Jx = J*x;
    Atx = A'*x;
    [sol,FVAL,EXITFLAG,OUTPUT,LAMBDA] = ...
        quadprog(H,[-Atx; 1],[JA -ones(m,1)],Jx,[ones(1,p) 0],1,[zeros(p,1); -Inf],[ones(p,1); 0],[],opts);
    
    as = sol(1:end-1);
    tau = sol(end);
    xb = A*as;
    d = xb - x;
    
    qd = 0.5*(d')*d;
    theta = tau + qd; 
    
    if theta > 0
       flag = 1;
       return
    end
    
    % stopping criterion
    %if abs(theta) < tol
    if abs(qd) < tol % theta - 0.5*(d')*d = tau
        if tau > -tol
            t0 = toc;
            [solfw,FVAL,EXITFLAG,OUTPUT,LAMBDA] = ...
            linprog([zeros(p,1); 1],[JA -ones(m,1)],Jx,[ones(1,p) 0],1,[zeros(p,1); -Inf],[ones(p,1); 0],[],opts);
            t1 = toc;
            tstop = tstop + (t1 - t0);
            thetafw = solfw(end); 

            % stopping criterion
            if abs(thetafw) < tol || thetafw > 0
                flag = 0;
                return
            else
               % fprintf('...falso positivo... \n');
            end
        end
    end
    
    
    %% Step-size 
    amax = 1;
    an = as;
    gamma = min(amax, -tau/(L*norm(d,2)^2)); %Line search
  
    %fprintf('k:%d  nnz(a):%d , gmax-gamma: %g , gamma: %g\n', k, nnz(a), amax - gamma, gamma);

    %% update iterate and convex combination coefficients
    x = x + gamma*d;
    aold = a;
    a = (1-gamma)*a + gamma*an;
    
    % clean up tiny negative coefficients
    a(a<tolp) = 0;
    a = a./sum(a);
    
    if norm(a - aold,1) < 5*tol
        flag = 2;
        return 
    end
    
    k = k+1;
    
    if nargout > 5
        xlist = [xlist; x'];
    end
    
    if (k > maxit)
        fprintf('CSD: maximum number of iterations reached!\n')
        flag=-1;
        return
    end
    
end
